/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import java.util.*;

/**
 * A manager for the Alerters.
 *
 * This class starts by loading all the alerters as specificed in the configuration.
 * These alerters should implement the PluginAlerter interface.
 *
 * This class is then simply a reference point for the Alerters to get hooks on
 * the various parts of the system that they require.
 *
 * @author  $Author: tdb $
 * @version $Id: AlerterManager.java,v 1.13 2004/08/01 10:40:40 tdb Exp $
 */
public class AlerterManager extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.13 $";
    
//---STATIC METHODS---
    
    /**
     * Return a reference to the single class.
     * Construct it if it does not already exist, otherwise just return the reference.
     */
    public synchronized static AlerterManager getInstance() {
        if (_instance == null){
            _instance = new AlerterManager();
        }
        return _instance;
    }

//---CONSTRUCTORS---
    
    /**
     * Constructs a new AlerterManager.
     * This initialises all the queues and loads
     * all the Alerters that have been specified in the configuration
     */
    private AlerterManager() {
        // set our name
        setName("client.AlerterManager");
        
        _logger.write(toString(), Logger.SYSINIT, "Initialising");
        _logger.write(toString(), Logger.SYSMSG, "Creating alerter pipeline for plugin alerters ...");
        
        // get the config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        
        // get the configuration for this plug-in setup
        String pluginsPackage, pluginsList;
        try {
            pluginsPackage = cp.getProperty(_name, "Alerter.PluginsPackage");
            pluginsList = cp.getProperty(_name, "Alerter.Plugins");
        } catch(PropertyNotFoundException e) {
            _logger.write(toString(), Logger.WARNING, "Unable to get required configuration, Alerter's will not be activated: "+e);
            // setting these will ensure we don't build a pipeline
            pluginsPackage = "";
            pluginsList = "";
        }
        
        StringTokenizer st = new StringTokenizer(pluginsList, ";");
        
        while(st.hasMoreTokens()) {
            String className = pluginsPackage + "." + st.nextToken() + _suffix;
            _logger.write(toString(), Logger.DEBUG, "Attempting to create plugin: "+className);
            
            // Create an instance of the specified PluginAlerter to include
            // within the alerterPipe.  Add it to the alerterPipeline
            try {
                PluginAlerter pa = (PluginAlerter)ClassLoader.getSystemClassLoader().loadClass(className).newInstance();
                _alerterPipeline.add(pa);
                _logger.write(toString(), Logger.DEBUG, "Added alerter: "+className+" ("+pa.getDescription()+")");
            }
            catch (InstantiationException e){
                _logger.write(toString(), Logger.ERROR, "Failed to instantiate "+className+" to the plugin alerter pipeline.");
                _logger.write(toString(), Logger.ERROR, e.getMessage());
            }
            catch (Exception e){
                _logger.write(toString(), Logger.ERROR, "Failed to add "+className+" to the plugin alerter pipeline.");
                _logger.write(toString(), Logger.ERROR, e.toString());
            }
        }
        _logger.write(toString(), Logger.SYSMSG, "The alerter pipeline has been set up with "+_alerterPipeline.size()+" plugin alerters.");
    }

//---PUBLIC METHODS---

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Allows alerters to obtain a reference
     * to the alerting queue so that they can 
     * process any alerts
     *
     * @return a reference to the alert queue
     */
    public Queue getQueue() {
        return ClientMain._alerterQueue;
    }

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * file name suffix for plugin alerter classes:
     */
    private final String _suffix = "__Alerter";
    
    /**
     * LinkedList for holding the PluginAlerter objects (the pipeline).
     */
    private LinkedList _alerterPipeline = new LinkedList();

//---STATIC ATTRIBUTES---

    /**
     * A reference to the single instance of this class
     */
    private static AlerterManager _instance;

}
