/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import java.text.*;
import java.util.*;

/**
 * Skeleton class for Alerters
 * This skeleton reads alerts from the alert queue designated 
 * to the Alerter, it then feeds the alerts to the sendAlert
 * method, which the extending class should implement.  The class
 * should then handle sending the message.  This class also has a 
 * method to allow formatting of messages.
 *
 * @author  $Author: tdb $
 * @version $Id: AlerterSkeleton.java,v 1.13 2004/08/01 10:40:40 tdb Exp $
 */
public abstract class AlerterSkeleton extends Thread implements PluginAlerter {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.13 $";

    /**
     * The default level to send alerts at
     */
    public final String DEFAULT_LEVEL = Alert.alertLevels[0];
    
    /**
     * If a configuration property is not 
     * configured, this will be used instead.
     */
    public final String NOT_CONFIGURED = "<NOT CONFIGURED>";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs and starts the alerter reading alerts
     */
    public AlerterSkeleton() {
        _logger.write(toString(), Logger.SYSINIT, "started.");
        this.start();
    }

//---PUBLIC METHODS---

    /**
     * Reads the next alert from the alerter queue for this
     * alerter.  It then looks up Alerter.<alerter name>.level in
     * the configuration to determine the lowest level of alert
     * the extending alerter should send.  If the alert is
     * above this level, or if the alert is an OK for an alert
     * escalation which did rise above this alerters level, it then
     * calls sendAlert to tell the alerter to send the alert.  If it
     * isn't above this level, it ignores the alert.
     */
    public void run() {
        while(_running) {
            try {
                Alert alert = (Alert) getQueue().get(getQueueId());
                String levelName;
                try {
                    levelName = _cp.getProperty(_name, "Alerter." + getFName() + ".level");
                } catch (PropertyNotFoundException e) {
                    levelName = DEFAULT_LEVEL;
                    _logger.write(toString(), Logger.WARNING, "Alerter." + getFName() + ".level value unavailable using default of " + levelName);
                }
                int level = StringUtils.getStringPos(levelName, Alert.alertLevels);
                // only send if it's equal (or above) our level
                if(((alert.getLevel() == 0) && (alert.getLastLevel() >= level)) || (alert.getLevel() >= level)) {
                    sendAlert(alert);
                }
            } catch (InvalidQueueException e) {
                _logger.write(this.toString(), Logger.ERROR, "Unable to get queue.");
            }
        }
    }
    
    /**
     * Sends an alert object using whatever transport
     * mechanism the extending class is implementing.
     *
     * @param alert the Alert to send
     */
    protected abstract void sendAlert(Alert alert);
    
    /**
     * Processes a message, replace key fields (eg. %level%) with
     * representative text from the given Alert object.
     *
     * @param message The message to process
     * @param alert The Alert object to get data from
     * @return the processed message
     */
    protected String processAlertMessage(String message, Alert alert) {
        // get some values
        String alertType = Alert.alertLevels[alert.getLevel()];
        String thresholdType = Alert.thresholdLevels[alert.getThreshold()];
        String timeSinceFirst = DateUtils.formatTime((System.currentTimeMillis() - alert.getInitialAlertTime())/1000, "%DAYS% days, %HOURS% hours, %MINS% mins, and %SECS% secs");
        String timeFirstOccured = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.UK).format(new Date(alert.getInitialAlertTime()));
        
        // replace fields in message
        message = StringUtils.replaceText(message, "%level%", alertType);
        message = StringUtils.replaceText(message, "%threshold%", thresholdType);
        message = StringUtils.replaceText(message, "%source%", alert.getSource());
        message = StringUtils.replaceText(message, "%value%", alert.getValue());
        message = StringUtils.replaceText(message, "%thresholdValue%", alert.getThresholdValue());
        message = StringUtils.replaceText(message, "%attributeName%", alert.getAttributeName());
        message = StringUtils.replaceText(message, "%timeTillNextAlert%",  DateUtils.getTimeString(Long.parseLong(alert.getTimeTillNextAlert())));
        message = StringUtils.replaceText(message, "%timeSinceFirstAlert%", timeSinceFirst);
        message = StringUtils.replaceText(message, "%timeOfFirstAlert%", timeFirstOccured);
        return message;
    }
    
    /**
     * Return the String representation of what the alerter does
     *
     * @return the description
     */
    public abstract String getDescription();


//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Gets hold of the Alerts queue from the AlerterManager.
     *
     * @return The Alerts queue
     */
    protected Queue getQueue() {
        return AlerterManager.getInstance().getQueue();
    }
    
    /**
     * Gets an id to an individual queue
     *
     * @return the id to the queue
     */
    protected int getQueueId() {
        if (_qID == -1) {
            _qID = getQueue().getQueue();
            _logger.write(toString(), Logger.DEBUG, "Assigned Queue - " + _qID);
        }
        return _qID;
    }

    /**
     * Returns the "friendly" name of this class. This
     * is simply an accessor for _name, required due to
     * inheritance issues with extending AlerterSkeleton.
     *
     * @return the friendly name
     */
    protected abstract String getFName();

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    protected String _name = "AlerterSkeleton";

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    protected Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the singleton configuration proxy
     * in use, through which alerter configuration can be obtained
     */
    protected ConfigurationProxy _cp = ConfigurationProxy.getInstance();
    
    /**
     * The ID of the queue the alerter will use.
     * Initially -1, but initialised on first use.
     */
    protected int _qID = -1;
    
    /**
     * The state of the alerter thread
     */
    protected boolean _running = true;

//---STATIC ATTRIBUTES---

}
