/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import java.util.HashMap;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * Skeleton class for Monitors
 * This skeleton reads packets from a queue designated 
 * by the extending class, it then feeds the data to the analysePacket
 * method, which the extending class should implement.  The class
 * should then handle the monitoring for that packet.
 *
 * @author  $Author: tdb $
 * @version $Id: MonitorSkeleton.java,v 1.19 2004/08/01 10:40:41 tdb Exp $
 */
public abstract class MonitorSkeleton extends Thread implements PluginMonitor {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.19 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs and start the monitor reading its data
     */
    public MonitorSkeleton() {
        _logger.write(toString(), Logger.SYSINIT, "started.");
        this.start();
    }

//---PUBLIC METHODS---

    /**
     * Obtains data from the monitors data queue and 
     * passes the packet to the analysePacket method.
     */
    public void run() {
        while(_running) {
            try {
                // check if monitoring is enabled for this host
                // if not - just don't bother analyse it's packets
                XMLPacket packet = (XMLPacket) getQueue().get(getQueueId());
                String source = packet.getParam("packet.attributes.machine_name");
                boolean enabled = checkBooleanConfig("Host." + source, "Monitor.enable");
                if(enabled) {
                    analysePacket(packet);
                }
		else {
		    _logger.write(this.toString(), Logger.DEBUG, "Skipping monitoring of host: " + source);
		} 
            } catch (InvalidQueueException e) {
                _logger.write(this.toString(), Logger.ERROR, "Unable to get queue.");
            }
        }
    }
    
    /**
     * Extending classes should override this method to
     * analyse the given packet for the attribute
     * they are responsible for.
     */
    protected abstract void analysePacket(XMLPacket packet);

    /**
     * Once a Monitor has determined which threshold the given data packet
     * is at, it should then call this method.  This method handles ALL 
     * the alerting logic to determine escalation of alerts.
     * If it decides an alert needs to be send, it will send one using fireAlert.
     *
     * @param newThreshold the threshold that has been determined by the monitor
     * @param attributeName the textual name of the attribute the monitor is responsible for
     * @param reg the register that holds the current alert state for the machine/attribute
     * @param source the source of the alert eg, hostname
     * @param currentValue the data value for the attribute
     */
    protected void processAlert(int newThreshold, String attributeName, Register reg, String source, String currentValue) {
        // decide what threshold level we're on, if we've changed, record that
        if (newThreshold != reg.getLastThresholdLevel()) {
            reg.setLastThresholdLevel(newThreshold);
        }
        // as long as this isn't a normal level
        if(reg.getLastThresholdLevel() != Alert.thresholdNORMAL) {
            // if the time since the last alert is more than the time for
            // its timeout, fire an alert, escalate the alert
            long timeout = reg.getLastAlertTimeout();
            if ((timeout > 0) && (reg.getTimeLastSent() > 0)) {
                if((System.currentTimeMillis() - reg.getTimeLastSent()) > timeout) {
                    int lastAlert = reg.getLastAlertLevel();
                    reg.escalateAlert();
                    reg.setTimeLastSent( System.currentTimeMillis());
                    reg.setLastAlertTimeout(reg.getAlertTimeout(reg.getLastAlertLevel()));
                    fireAlert(reg, lastAlert, source, currentValue, attributeName);
                }
            // if we don't have a timeout configured...we got STRAIGHT to the next level
            } else {
                int lastAlert = reg.getLastAlertLevel();
                reg.escalateAlert();
                reg.setTimeLastSent( System.currentTimeMillis());
                reg.setLastAlertTimeout(reg.getAlertTimeout(reg.getLastAlertLevel()));
                fireAlert(reg, lastAlert, source, currentValue, attributeName);
            }
                
        // we must be on ok, check the timeout value for this
        } else {
            // if we were on an OK alert before, then we don't do anything
            // but if we weren't we only set OK, once the timout of the last 
            // alert has occourd
            if (reg.getLastAlertLevel() != Alert.alertOK) {
                long timeout = reg.getLastAlertTimeout();
                if ((timeout > 0) && (reg.getTimeLastSent() > 0)) {
                    if ((System.currentTimeMillis() - reg.getTimeLastSent()) > timeout) {
                        int lastAlert = reg.getLastAlertLevel();
                        reg.setLastAlertLevel(Alert.alertOK);
                        reg.setTimeLastSent(System.currentTimeMillis());
                        reg.setLastAlertTimeout(timeout);
                        fireAlert(reg, lastAlert,  source, currentValue, attributeName);
                    }
                }
            }
        }
    }

    /**
     * Return the String representation of what the alerter does
     *
     * @return the description
     */
    public abstract String getDescription();

    /**
     * Checks a boolean configuration variable.
     *
     * @param source the configuration to look up
     * @param name the key to look up
     */
    protected boolean checkBooleanConfig(String source, String name) {
        boolean enabled = true;
        try {
            int e = Integer.parseInt(_cp.getProperty(source, name));
            enabled = (e == 1);
        } catch (PropertyNotFoundException e) {
            enabled = false;
        } catch (NumberFormatException e) {
            enabled = false;
        }
        return enabled;
    }

//---PRIVATE METHODS---

    /**
     * Fires an alert.  This creates a new Alert object
     * and populates it with the given alert information.
     * It then adds the alert to the Alerter queue.
     *
     * This method should only be called by the processAlert method.
     *
     * @param reg the register holding the state values for the alert
     * @param source the source of the alert eg, hostname
     * @param currentValue the data value for the attribute
     * @param attributeName the textual name of the attribute the alert is for
     */
    private void fireAlert(Register reg, int lastAlert, String source, String currentValue, String attributeName) {
        int alertLevel = reg.getLastAlertLevel();
        int thresholdLevel = reg.getLastThresholdLevel();
        String thresholdValue = String.valueOf(reg.getThreshold(thresholdLevel));
        String timeout = String.valueOf(reg.getAlertTimeout(reg.getLastAlertLevel()) / 1000);
        // ensures we display a nice thing if its -1.0
        if (thresholdValue.equals("-1.0")) {
            thresholdValue = "-";
        }
        if (alertLevel == Alert.alertOK) {
            timeout = "0";
        }
        Alert alert = new Alert(alertLevel, lastAlert, thresholdLevel, source, thresholdValue, currentValue, attributeName, timeout, reg.getInitialAlertTime());
        _alerterQueue.add(alert);
        _logger.write(toString(), Logger.DEBUG, "Fired alert for source:" + source + " at alert level:" + Alert.alertLevels[alertLevel] + " on:" + attributeName + " for threshold level:" + Alert.thresholdLevels[thresholdLevel] + " at:" +  currentValue + " exceeding threshold of:" +thresholdValue + " next alert sent in:" + timeout + "secs");
    }

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Obtain the queue which contains the data
     * the Monitor is reading.
     * eg, MonitorManager.getInstance().getDataQueue()
     */
    protected abstract Queue getQueue();
    
    /**
     * Create a queue ID on the feeding
     * data queue
     */
    protected int getQueueId() {
        if (_qID == -1) {
            _qID = getQueue().getQueue();
            _logger.write(toString(), Logger.DEBUG, "Assigned Queue - " + _qID);
        }
        return _qID;
    }

//---ATTRIBUTES---

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    protected Logger _logger = ReferenceManager.getInstance().getLogger();

    /**
     * A reference to the Alerter queue, into which
     * all new alerts will be placed.
     */
    protected Queue _alerterQueue = AlerterManager.getInstance().getQueue();

    /**
     * A reference to the configuration proxy in use
     */
    private ConfigurationProxy _cp = ConfigurationProxy.getInstance();

    /**
     * The ID of the queue the monitor will use.
     * Initially -1, but initialised on first use.
     */
    protected int _qID = -1;
    
    /**
     * The state of the alerter thread
     */
    protected boolean _running = true;

//---STATIC ATTRIBUTES---

}
